"use strict";
/**
 * Reclass doc generator
 *
 * @author Jiri Hybek <jiri@hybek.cz>
 * @license Apache-2.0 (c) 2017 Jiri Hybek
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Parse mode
 */
var PARSE_MODE;
(function (PARSE_MODE) {
    PARSE_MODE[PARSE_MODE["OBJECT"] = 0] = "OBJECT";
    PARSE_MODE[PARSE_MODE["LIST"] = 1] = "LIST";
    PARSE_MODE[PARSE_MODE["VALUE"] = 2] = "VALUE";
})(PARSE_MODE || (PARSE_MODE = {}));
/**
 * Parse error class
 */
var ParserError = (function (_super) {
    __extends(ParserError, _super);
    function ParserError(src, line, col, token, message) {
        var _this = this;
        var lines = src.split("\n");
        var code = (lines[line - 1] || "") + lines[line].substr(0, col - 1) + ">>" + lines[line].substr(col, 1) + "<<" + lines[line].substr(col + 1) + (lines[line + 1] || "");
        _this = _super.call(this, "YAML parse error at" + line + ":" + col + ": " + message + "\n\n" + code) || this;
        _this.src = src;
        _this.col = col;
        _this.line = line;
        _this.token = token;
        return _this;
    }
    return ParserError;
}(Error));
exports.ParserError = ParserError;
/**
 * YAML parser class
 */
var YamlParser = (function () {
    function YamlParser() {
    }
    /**
     * Parse yaml from string
     *
     * @param src Source
     */
    YamlParser.prototype.parse = function (src) {
        var index = 0;
        var line = 0;
        var col = 0;
        var indent = 0;
        var indentSize = 0;
        var scope = [];
        var mode = PARSE_MODE.OBJECT;
        var quotes = null;
        var buffer = "";
        var lastMatch = null;
        var comment = [];
        var token = {
            type: "root",
            value: {}
        };
        var nest = function () {
            scope.push({
                m: mode,
                t: token,
                q: quotes
            });
        };
        var unest = function () {
            var ctx = scope.pop();
            if (!ctx)
                return;
            mode = ctx.m;
            token = ctx.t;
            quotes = ctx.q;
        };
        var error = function (msg) {
            throw new ParserError(src, line, col, token, msg);
        };
        var eat = function (pattern, toBuffer) {
            if (toBuffer === void 0) { toBuffer = false; }
            var haystack = src.substr(index);
            var match = pattern.exec(haystack);
            if (match) {
                index += String(match[0]).length;
                col += String(match[0]).length;
                lastMatch = match[0];
                if (toBuffer)
                    buffer += lastMatch;
                return true;
            }
            else {
                lastMatch = null;
            }
            return false;
        };
        var skip = function (count) {
            index += count;
            col += count;
        };
        //Parse
        while (index < src.length) {
            //Check quotes
            if (eat(/^["|']/)) {
                //End quote
                if (quotes == lastMatch)
                    quotes = null;
                else if (quotes === null)
                    quotes = lastMatch;
                else
                    buffer += lastMatch;
                continue;
            }
            //Check comment
            if (eat(/^#/)) {
                eat(/^.+?(?=\\n)/);
                if (lastMatch)
                    comment.push(lastMatch);
                continue;
            }
            //Parse object
        }
        return token;
    };
    return YamlParser;
}());
exports.YamlParser = YamlParser;
